#!/usr/bin/env perl

#
# iEȗ͂̌ʂ\ (kuro@i.kyoto-u.ac.jp)
#
# Usage:
# 1. juman | knp -case -tab > aaa.knp
# 2. perl print_pa.pl [--tree|--merged] [--all] aaa.knp
#
# --tree   \؂\
# --merged \؂̉Eɏqꍀ\\
# --all    TϖȂǂɑ΂qꍀ\\
#
# (vgAEg̍ۂ́C
#    perl print_pa.pl aaa.knp | e2ps -size 7.0 -l | lpr 
#  ƂƂꂢɏo͂ł܂)

use strict;
use KNP::File;
use encoding 'shiftjis';
use Getopt::Long;

our %opt;
our %ForcePrintCases = ('K' => 1, '' => 1, 'j' => 1); # [㖼\i

&GetOptions(\%opt, 'tree', 'merged', 'all');
$opt{tree} = 1 if $opt{merged};

my $knp = new KNP::File($ARGV[0]);
my $result;
my @result_lst;


sub usage
{
    print "$0 [--tree|--merged] [--all] file.knp";
    exit 1;
}

#######################
sub tag_string
#######################
{
    my ($tag, $base_flag) = @_;
    my (@ret, $last_base);

    foreach my $mrph ($tag->mrph) {
	last if ($mrph->hinsi eq "" ||
		 $mrph->hinsi eq "莌" ||
		 $mrph->hinsi eq "");
	push(@ret, $mrph->midasi);
	$last_base = $mrph->genkei;
    }

    if ($base_flag) {
	pop(@ret);
	push(@ret, $last_base); # Ōゾ^
    }

    return join('', @ret);
}

#######################
sub compound_noun_string
#######################
{
    my ($tag, $base_flag) = @_;
    my ($string);

    for my $c ($tag->child) {
	if ($c->id + 1 == $tag->id && # ƂȂ
	    $c->fstring =~ /<W:ߓ>/) {
	    $string = &compound_noun_string($c);
	    last;
	}
    }

    return $string . &tag_string($tag, $base_flag);
}

#######################
sub process_case_analysis_result
#######################
{
    my ($bnst, $tag, $case, $flag, $hyouki, $tid, $sid);
    my (@case_result, $atag, $print_string, $t_string, $type, @ret);

    for $bnst ($result->bnst) {
	for $tag ($bnst->tag) {
	    next if !$opt{all} && 
		($tag->fstring =~ /\<pi/ ||	# TϖȂǂ̓XLbv
		 $tag->fstring =~ /\<x:A-\>/);	# アp̓XLbv

	    @case_result = ();
	    $print_string ="";

	    # <i͌::16:K/N/A/1;/C/̑/8;j/O/Ag/6/1/?>
	    # C: ڌW(i)
	    # N: ڌW(i񖾎)
	    # O: ȗ
	    # D: w̎wΏ

	    # ʂ\pi肽΁C
	    #
	    # $tag->fstring !~ /\<x:A/ &&
	    #
	    # ̂悤ɏ΂悢

	    if ($tag->fstring =~ /\<i͌:(.+?):([^:]+(?::[A-Z]+)?\d+):(.+?)\>/) {

		foreach (split(/;/, $3)) {
		    $tid=0;$sid=0;
		    ($case, $flag, $hyouki, $tid, $sid) = split(/\//, $_);
		    if ($flag eq "C" || $flag eq "N") {
			$case_result[$tid]{case} = $case;
			$t_string = &compound_noun_string(($result->tag)[$tid]);
			$type = "";
			$print_string .= " $t_string$type:$case";
		    }
		    elsif ($flag eq "O") {
			$t_string = &compound_noun_string(($result_lst[$#result_lst - $sid]->tag)[$tid]);
			$type = "()";
			$print_string .= " $t_string$type:$case";
		    }
		    elsif ($flag eq "D") {
			$t_string = &compound_noun_string(($result_lst[$#result_lst - $sid]->tag)[$tid]);
			$type = "(w)";
			$print_string .= " $t_string$type:$case";
		    }
		    elsif (defined($ForcePrintCases{$case})) {
			$print_string .= " :$case";
		    }
		}

		# ˑ\̎qCeŊi͌ʂɓĂȂ

		for $atag ($tag->child) {
		    if (!(exists $case_result[$atag->id]{case}) && 
			$atag->fstring !~ /\<ȗ͑Ώێw\>/ && 
			$atag->fstring =~ /\<̌\>/) {
			$print_string .= (" " . &compound_noun_string($atag) . ":--");
		    }
		}
		if (defined($atag = $tag->parent)) { # ȀC
		    if (!(exists $case_result[$atag->id]{case}) && 
			$atag->fstring !~ /\<ȗ͑Ώێw\>/ && 
			$atag->fstring =~ /\<W:(?:Ai|ߓ)\>/) {
			$print_string .= (" " . &compound_noun_string($atag) . ":--");
		    }
		}

		if ($opt{merged}) {
		    $tag->pstring($print_string);
		}

		$print_string = &tag_string($tag, 1) . $print_string;
		push(@ret, $print_string);
	    }
	}
    }

    return @ret;
}

#######################
# MAIN
#######################

while ($result = $knp->each()) {
    print $result->comment;

    push(@result_lst, $result);
    my @ca_result = &process_case_analysis_result;

    # ؂̕\
    if ($opt{tree}) {
	$result->draw_tag_tree;
	print "\n";
    }

    # i͌ʂ̕\
    unless ($opt{merged}) { # merged̂Ƃ͂łtree̒ɕ\Ă
	print join("\n", @ca_result), "\n";
	print "\n";
    }
}
